@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -180,7 +180,7 @@ static void select_list_box (void)
 	return;
      }
    
-   ret = slrn_select_list_mode (title, argc, argv, active - 1, NULL);
+   ret = slrn_select_list_mode (title, argc, argv, active - 1, 1, NULL);
    if (ret == -1)
      {
 	SLang_push_string ("");
@@ -565,7 +565,7 @@ static void undefinekey (char *key, char *map) /*{{{*/
 #define PROMPT_VARIABLE 3
 static void generic_read_mini (int mode, char *prompt, char *dfl, char *init) /*{{{*/
 {
-   char str[256];
+   char str[SLRL_DISPLAY_BUFFER_SIZE];
    int ret;
 
    strncpy (str, init, sizeof (str));
@@ -864,7 +864,7 @@ static char *extract_current_article_header (char *h) /*{{{*/
 
 /*}}}*/
 
-static char *article_as_string (void) /*{{{*/
+static char *generic_article_as_string (int use_orig) /*{{{*/
 {
    unsigned int len;
    Slrn_Article_Line_Type *l;
@@ -873,7 +873,7 @@ static char *article_as_string (void) /*{{{*/
    if (Slrn_Current_Article == NULL)
      return "";
 
-   l = Slrn_Current_Article->lines;
+   l = use_orig ? Slrn_Current_Article->raw_lines : Slrn_Current_Article->lines;
 
    len = 0;
    while (l != NULL)
@@ -897,7 +897,7 @@ static char *article_as_string (void) /*{{{*/
 	return NULL;
      }
    
-   l = Slrn_Current_Article->lines;
+   l = use_orig ? Slrn_Current_Article->raw_lines : Slrn_Current_Article->lines;
    s1 = s;
    
    while (l != NULL)
@@ -917,7 +917,18 @@ static char *article_as_string (void) /*{{{*/
    *s1 = 0;
    return s;
 }
+/*}}}*/
 
+static char *article_as_string (void) /*{{{*/
+{
+   return generic_article_as_string (0);
+}
+/*}}}*/
+
+static char *raw_article_as_string (void) /*{{{*/
+{
+   return generic_article_as_string (1);
+}
 /*}}}*/
 
 static void sort_articles (void) /*{{{*/
@@ -930,6 +941,64 @@ static void sort_articles (void) /*{{{*/
 
 /*}}}*/
 
+static char *article_cline_as_string (void) /*{{{*/
+{
+   if (-1 == check_article_mode () ||
+       (Slrn_Current_Article == NULL))
+     return "";
+   return Slrn_Current_Article->cline->buf;
+}
+/*}}}*/
+
+static int article_cline_number (void) /*{{{*/
+{
+   if (-1 == check_article_mode ()) return 0;
+   return (int)slrn_art_cline_num();
+}
+/*}}}*/
+
+static int article_count_lines (void) /*{{{*/
+{
+   if (-1 == check_article_mode ()) return 0;
+   return (int)slrn_art_count_lines();
+}
+/*}}}*/
+
+static int article_goto_line (int *num) /*{{{*/
+{
+   int offset;
+   if ((-1 == check_article_mode ())
+       || (*num <= 0))
+     return 0;
+   if (0 != (offset = *num - slrn_art_cline_num()))
+     {
+	if (offset > 0) slrn_art_linedn_n ((unsigned int)offset);
+	else slrn_art_lineup_n ((unsigned int)-offset);
+     }
+   return (int)slrn_art_cline_num();
+}
+/*}}}*/
+
+static int article_line_up (int *num) /*{{{*/
+{
+   if ((-1 == check_article_mode ())
+       || (*num <= 0))
+     return 0;
+   
+   return slrn_art_lineup_n ((unsigned int)*num);
+}
+/*}}}*/
+
+static int article_line_down (int *num) /*{{{*/
+{
+   if ((-1 == check_article_mode ())
+       || (*num <= 0))
+     return 0;
+   
+   return slrn_art_linedn_n ((unsigned int)*num);
+}
+/*}}}*/
+
 /*}}}*/
 
 /*{{{ Article Mode Header Functions */
@@ -940,6 +1009,10 @@ static int Interp_Header_Read = HEADER_READ;
 static int Interp_Header_Tagged = HEADER_TAGGED;
 static int Interp_Header_High_Score = HEADER_HIGH_SCORE;
 static int Interp_Header_Low_Score = HEADER_LOW_SCORE;
+static int Interp_Attr_Blink = SLTT_BLINK_MASK;
+static int Interp_Attr_Bold = SLTT_BOLD_MASK;
+static int Interp_Attr_Rev = SLTT_REV_MASK;
+static int Interp_Attr_Uline = SLTT_ULINE_MASK;
 #ifdef HAVE_SETLOCALE
 # ifdef LC_TIME
 static int Interp_Lc_Time = LC_TIME;
@@ -984,13 +1057,26 @@ static int thread_size (void)
    if (check_article_mode ()) return -1;
    return (int) slrn_thread_size (Slrn_Current_Header);
 }
-   
+
+static int has_parent (void)
+{
+   if (check_article_mode ()) return -1;
+   return (Slrn_Current_Header->parent == NULL) ? 0 : 1;
+}
+
 static int is_thread_collapsed (void)
 {
    if (check_article_mode ()) return -1;
    return slrn_is_thread_collapsed (Slrn_Current_Header);
 }
 
+static int header_cursor_pos (void) /*{{{*/
+{
+   if (check_article_mode ()) return -1;
+   return slrn_header_cursor_pos ();
+}
+/*}}}*/
+
 static int header_down (int *num) /*{{{*/
 {
    if ((-1 == check_article_mode ())
@@ -1018,7 +1104,7 @@ static int header_next_unread (void)
    if (-1 == check_article_mode ())
      return -1;
    
-   return slrn_next_unread_header ();
+   return slrn_next_unread_header (1);
 }
 
 static int get_header_flags (void) /*{{{*/
@@ -1042,6 +1128,29 @@ static void set_header_flags (int *flagsp) /*{{{*/
 
 /*}}}*/
 
+static int get_body_status (void) /*{{{*/
+{
+   if ((-1 == check_article_mode ())
+       || (Slrn_Current_Header == NULL))
+     return -1;
+   if (Slrn_Current_Header->flags & HEADER_REQUEST_BODY)
+     return 2;
+   return (Slrn_Current_Header->flags & HEADER_WITHOUT_BODY) ? 1 : 0;
+}
+/*}}}*/
+
+static void request_body (int *mode) /*{{{*/
+{
+   if ((-1 == check_article_mode ())
+       || (Slrn_Current_Header == NULL))
+     return;
+   if (*mode)
+     slrn_request_header (Slrn_Current_Header);
+   else
+     slrn_unrequest_header (Slrn_Current_Header);
+}
+/*}}}*/
+
 static int get_header_number (void)
 {
    if ((-1 == check_article_mode ())
@@ -1251,7 +1360,12 @@ static void hide_current_group (void)
 
 static void set_color (char *obj, char *fg, char *bg)
 {
-   slrn_set_object_color (obj, fg, bg);
+   slrn_set_object_color (obj, fg, bg, 0);
+}
+
+static void set_color_attr (char *obj, char *fg, char *bg, int *attr)
+{
+   slrn_set_object_color (obj, fg, bg, (SLtt_Char_Type) *attr);
 }
 
 static char *get_fg_color (char *name)
@@ -1339,7 +1453,8 @@ static void replace_article_cmd (char *str)
        || (Slrn_Current_Header == NULL))
      return;
 
-   (void) slrn_string_to_article (str);
+   if (-1 == slrn_string_to_article (str))
+     slrn_error (_("Could not replace article with given string."));
 }
 
 static int is_article_visible (void)
@@ -1386,6 +1501,12 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("reset_prefix_arg", reset_prefix_arg, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SI("locate_header_by_msgid", locate_header_by_msgid, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("article_as_string", article_as_string, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_0("article_cline_as_string", article_cline_as_string, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_0("article_cline_number", article_cline_number, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("article_count_lines", article_count_lines, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_I("article_goto_line", article_goto_line, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_I("article_line_down", article_line_down, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_I("article_line_up", article_line_up, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("bsearch_article", bsearch_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("call", slrn_call_command, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("collapse_thread", collapse_thread, SLANG_VOID_TYPE),
@@ -1396,6 +1517,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_S("extract_article_header", extract_article_header, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_S("extract_displayed_article_header", extract_current_article_header, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_S("get_bg_color", get_bg_color, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_0("get_body_status", get_body_status, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("get_fg_color", get_fg_color, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_0("get_group_flags", get_group_flags, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("get_grouplens_score", get_grouplens_score, SLANG_INT_TYPE),
@@ -1414,6 +1536,8 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_S("group_search", group_search, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("group_unread", get_group_unread_count, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("group_up_n", group_up_n, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("has_parent", has_parent, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("header_cursor_pos", header_cursor_pos, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("header_down", header_down, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("header_next_unread", header_next_unread, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("header_up", header_up, SLANG_INT_TYPE),
@@ -1429,6 +1553,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("popup_window", popup_window, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("prev_tagged_header", slrn_prev_tagged_header, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("quit", quit, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("raw_article_as_string", raw_article_as_string, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_S("re_bsearch_article", re_bsearch_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("re_bsearch_author", re_author_search_backward, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("re_bsearch_subject", re_subject_search_backward, SLANG_INT_TYPE),
@@ -1443,6 +1568,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_SI("read_mini_integer", read_mini_integer, SLANG_INT_TYPE),
    MAKE_INTRINSIC_SS("register_hook", slrn_register_hook_by_name, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("reload_scorefile", reload_scorefile, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_I("request_body", request_body, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("save_current_article", save_current_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("search_article", search_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("search_article_first", search_article_first, SLANG_INT_TYPE),
@@ -1452,6 +1578,8 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("get_article_window_size", get_article_window_size, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("set_article_window_size", set_article_window_size, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SSS("set_color", set_color, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_4("set_color_attr", set_color_attr, SLANG_VOID_TYPE,
+		    SLANG_STRING_TYPE, SLANG_STRING_TYPE, SLANG_STRING_TYPE, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("set_group_flags", set_group_flags, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("hide_current_group", hide_current_group, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("set_header_flags", set_header_flags, SLANG_VOID_TYPE),
@@ -1485,6 +1613,10 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
 
 static SLang_Intrin_Var_Type Intrin_Vars [] = /*{{{*/
 {
+   MAKE_VARIABLE("ATTR_BLINK", &Interp_Attr_Blink, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("ATTR_BOLD", &Interp_Attr_Bold, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("ATTR_REV", &Interp_Attr_Rev, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("ATTR_ULINE", &Interp_Attr_Uline, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("GROUPS_DIRTY", &Slrn_Groups_Dirty, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("GROUP_NEW_GROUP_FLAG", &Interp_Group_New_Group_Flag, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("GROUP_UNSUBSCRIBED", &Interp_Group_Unsubscribed, SLANG_INT_TYPE, 1),
@@ -1522,10 +1654,12 @@ int slrn_init_slang (void) /*{{{*/
    Slrn_Use_Slang = 0;
    if ((-1 == SLang_init_slang ())
        || (-1 == SLang_init_slmath ())
-#ifdef __unix__
-       || (-1 == SLang_init_slunix ())
-#endif
-       || (-1 == SLang_init_slfile ())
+       || (-1 == SLang_init_posix_process ())
+       || (-1 == SLang_init_posix_dir ())
+       || (-1 == SLang_init_stdio ())
+       || (-1 == SLang_init_posix_io ())
+       || (-1 == SLang_init_ospath ())
+       || (-1 == SLang_init_slassoc ())
 #if SLANG_VERSION >= 10400
        || (-1 == SLang_init_import ()) /* enable dynamic linking */
 #endif
