@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -486,7 +486,7 @@ void slrn_custom_printf (char *fmt, PRINTF_CB cb, void *param, /*{{{*/
    
    while ((ch = *fmt) != 0)
      {
-	char *s;
+	char *s, *p;
 	int color = def_color;
 	int len = -1, field_len = -1;
 	int justify, spaces = 0;
@@ -600,6 +600,8 @@ void slrn_custom_printf (char *fmt, PRINTF_CB cb, void *param, /*{{{*/
 	  slrn_set_color (color);
 
 	if (len == -1) len = strlen (s);
+	if (NULL != (p = slrn_strchr (s, '\n')))
+	  len -= strlen (p);
 	if (field_len != -1)
 	  {
 	     int i;
@@ -675,7 +677,7 @@ unsigned int slrn_toggle_format (char **formats, unsigned int cur) /*{{{*/
 	retval = (unsigned int) *Slrn_Prefix_Arg_Ptr % SLRN_MAX_DISPLAY_FORMATS;
 	Slrn_Prefix_Arg_Ptr = NULL;
      }
-   else retval++;
+   else retval = (retval + 1) % SLRN_MAX_DISPLAY_FORMATS;
    
    while ((retval != cur) && (formats[retval] == NULL))
      retval = (retval + 1) % SLRN_MAX_DISPLAY_FORMATS;
@@ -836,7 +838,7 @@ void slrn_make_home_filename (char *name, char *file, size_t n) /*{{{*/
 
 /*}}}*/
 
-int slrn_make_home_dirname (char *name, char *dir, size_t n) /*{{{*/
+void slrn_make_home_dirname (char *name, char *dir, size_t n) /*{{{*/
 {
    /* This needs modified to deal with VMS directory syntax */
 #ifndef VMS
@@ -850,7 +852,7 @@ int slrn_make_home_dirname (char *name, char *dir, size_t n) /*{{{*/
    if (NULL != slrn_strchr (name, ':'))
      {
 	slrn_strncpy (dir, name, n);
-	return 0;
+	return;
      }
    home = getenv ("HOME");
    *dir = 0;
@@ -913,7 +915,7 @@ int slrn_make_home_dirname (char *name, char *dir, size_t n) /*{{{*/
      }
 #endif /* VMS */
    
-   return 0;
+   return;
 }
 
 /*}}}*/
@@ -1040,7 +1042,7 @@ FILE *slrn_open_home_file (char *name, char *mode, char *file, /*{{{*/
      }
 #else
    (void) create_flag;
-#endif	
+#endif
 
    if (2 == slrn_file_exists (file)) /* don't open directories */
      return NULL;
@@ -1069,6 +1071,7 @@ VFILE *slrn_open_home_vfile (char *name, char *file, size_t n)
 int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char *subject) /*{{{*/
 {
    char *buf;
+   char fcc_file[SLRN_MAX_PATH_LEN];
 #if defined(IBMPC_SYSTEM)
    char outfile [SLRN_MAX_PATH_LEN];
 #endif
@@ -1100,14 +1103,11 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
    if ((Slrn_Use_Mime & MIME_ARCHIVE) && /* else: do it later */
        !Slrn_Editor_Uses_Mime_Charset)
      slrn_chmap_fix_file (file, 0);
-   
-   if (-1 == slrn_save_file_to_mail_file (file, Slrn_Save_Replies_File, NULL))
-     return -1;
-   
+
    if (!(Slrn_Use_Mime & MIME_ARCHIVE) &&
        !Slrn_Editor_Uses_Mime_Charset)
      slrn_chmap_fix_file (file, 0);
-   
+
 #ifdef VMS
    buf = slrn_strdup_printf ("%s\"%s\"", MAIL_PROTOCOL, to);
    vms_send_mail (buf, subject, file);
@@ -1121,13 +1121,21 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 # if SLRN_HAS_MIME
    if (Slrn_Use_Mime & MIME_DISPLAY)
      {
-	FILE *fp, *pp;
+	FILE *fp, *pp, *fcc_fp;
 	int header = 1;
 	char line[1024];
-	
-	fp = fopen (file, "r");
-	if (fp == NULL) return (-1);
-	
+
+	if ((fp = fopen (file, "r")) == NULL)
+	     return (-1);
+
+	fcc_fp = slrn_open_tmpfile (fcc_file, sizeof (fcc_file));
+
+	if (fcc_fp == NULL)
+	  {
+	     slrn_fclose (fp);
+	     return (-1);
+	  }
+
 	slrn_mime_scan_file (fp);
 #  if defined(IBMPC_SYSTEM)
 	pp = slrn_open_tmpfile (outfile, sizeof (outfile));
@@ -1137,27 +1145,41 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 	if (pp == NULL)
 	  {
 	     slrn_fclose (fp);
+	     slrn_fclose (fcc_fp);
 	     return (-1);
 	  }
-	
+
 	while (fgets (line, sizeof(line), fp) != NULL)
 	  {
 	     unsigned int len = strlen (line);
-	     
+
 	     if (len == 0) continue;
 	     len--;
-	     
+
 	     if (line [len] == '\n') line [len] = 0;
-	     
+
 	     if (header)
 	       {
+		  if ((( line[0] == 'R') || (line[0]== 'r'))
+		      && (0 == slrn_case_strncmp ((unsigned char *)"References: ",
+						  (unsigned char *)line,
+						  12)))
+		    {
+		       (void) slrn_add_references_header (pp, line);
+		       (void) slrn_add_references_header (fcc_fp, line);
+		       continue;
+		    }
+
 		  if (line[0] == 0)
 		    {
 		       header = 0;
-		       
+
 		       slrn_add_date_header (pp);
 		       slrn_mime_add_headers (pp);
 		       fp = slrn_mime_encode (fp);
+
+		       slrn_add_date_header (fcc_fp);
+		       slrn_mime_add_headers (fcc_fp);
 		    }
 		  slrn_mime_header_encode (line, sizeof(line));
 	       }
@@ -1168,9 +1190,12 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 	       {
 		  fputs (line, pp);
 		  putc('\n', pp);
+		  fputs (line, fcc_fp);
+		  putc('\n', fcc_fp);
 	       }
 	  }
 	slrn_fclose (fp);
+	slrn_fclose (fcc_fp);
 #  if defined(IBMPC_SYSTEM)
 	slrn_fclose (pp);
 	buf = slrn_strdup_strcat (Slrn_SendMail_Command, " ", outfile, NULL);
@@ -1193,6 +1218,12 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
      }
 #endif /* NOT VMS */
    slrn_message (_("Sending...done"));
+
+   if (-1 == slrn_save_file_to_mail_file (fcc_file, Slrn_Save_Replies_File))
+     return -1;
+
+   (void) slrn_delete_file (fcc_file);
+
    return 0;
 }
 
@@ -1561,14 +1592,14 @@ static SLang_RLine_Info_Type *init_readline (void) /*{{{*/
    if (rli == NULL)
      return NULL;
    
-   if (NULL == (buf = (unsigned char *) slrn_malloc (256, 0, 1)))
+   if (NULL == (buf = (unsigned char *) slrn_malloc (SLRL_DISPLAY_BUFFER_SIZE, 0, 1)))
      {
 	SLFREE (rli);
 	return NULL;
      }
 
    rli->buf = buf;
-   rli->buf_len = 255;
+   rli->buf_len = SLRL_DISPLAY_BUFFER_SIZE - 1;
    rli->tab = 8;
    rli->dhscroll = 20;
    rli->getkey = SLang_getkey;
@@ -1885,7 +1916,7 @@ static void generic_mini_complete (int cycle) /*{{{*/
 	     qsort_fun (argv, argc, sizeof (char *), strpcmp);
 	     
 	     sel = slrn_select_list_mode (_("Possible completions"), argc,
-					  argv, 0, NULL);
+					  argv, 0, 1, NULL);
 	     slrn_update_screen ();
 	     if (sel != -1)
 	       {
@@ -1994,13 +2025,13 @@ SLKeymap_Function_Type Slrn_Custom_Readline_Functions [] =
    A_KEY(NULL, NULL)
 };
 
-/* str needs to have enough space for 256 characters */
+/* str needs to have enough space for SLRL_DISPLAY_BUFFER_SIZE characters */
 static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag, 
 			       int no_echo, int point) /*{{{*/
 {
    int i;
    int tt_init_state;
-   char prompt_buf[256];
+   char prompt_buf[SLRL_DISPLAY_BUFFER_SIZE];
    unsigned int len;
    int save_slang_error;
    
@@ -2163,7 +2194,7 @@ int slrn_read_integer (char *prompt, int *dflt, int *np) /*{{{*/
 {
    char sdfl_buf[32];
    char *sdfl = NULL;
-   char str[256];
+   char str[SLRL_DISPLAY_BUFFER_SIZE];
    int n;
    
    if (dflt != NULL)
@@ -2404,7 +2435,7 @@ void slrn_get_mouse_rc (int *rp, int *cp) /*{{{*/
 
 void slrn_evaluate_cmd (void) /*{{{*/
 {
-   char buf[256];
+   char buf[SLRL_DISPLAY_BUFFER_SIZE];
    
    *buf = '\0';
    if (slrn_read_input ("S-Lang> ", NULL, buf, 0, 0) > 0)
@@ -2560,13 +2591,19 @@ void slrn_get_user_info (void) /*{{{*/
 #endif
    
    /* Fill in what is assumed to be non-NULL by rest of program. */
-   Slrn_User_Info.followup_string = slrn_safe_strmalloc (_("In article %m, %r wrote:"));
-   Slrn_User_Info.reply_string = slrn_safe_strmalloc (_("In %n, you wrote:"));
+   
+   /* no-c-format tells gettext that the following strings do not
+    * need to be checked as if they were passed to printf. */
 
-   Slrn_CC_Followup_Message =
+   Slrn_User_Info.followup_string =   /* xgettext:no-c-format */
+     slrn_safe_strmalloc (_("On %D, %r <%f> wrote:"));
+   Slrn_User_Info.reply_string =      /* xgettext:no-c-format */
+     slrn_safe_strmalloc (_("In %n, you wrote:"));
+   Slrn_User_Info.followupto_string = /* xgettext:no-c-format */
+     slrn_safe_strmalloc (_("[\"Followup-To:\" header set to %n.]"));
+
+   Slrn_CC_Post_Message =         /* xgettext:no-c-format */
      slrn_safe_strmalloc (_("[This message has also been posted to %n.]"));
-   Slrn_CC_Post_Message =
-     slrn_safe_strmalloc (_("[This message has also been posted.]"));
    
    /* Now get default values for rest. */
    host = get_hostname ();
@@ -2689,6 +2726,9 @@ void slrn_get_user_info (void) /*{{{*/
    
    Slrn_User_Info.signature = slrn_safe_strmalloc (SLRN_SIGNATURE_FILE);
    
+#if SLRN_HAS_CANLOCK
+   Slrn_User_Info.cancelsecret = slrn_safe_strmalloc ("");
+#endif
 #ifdef SLRN_SENDMAIL_COMMAND
    Slrn_SendMail_Command = slrn_safe_strmalloc (SLRN_SENDMAIL_COMMAND);
 #endif
@@ -2790,8 +2830,8 @@ char *slrn_make_from_string (void)
 	SLang_free_slstring (msg);
 	/* Drop through to default */
      }
-   msg = NULL;
 #endif
+   msg = NULL;
 
    if (make_localpart (Slrn_User_Info.username, localpart, sizeof (localpart)))
      msg = _("Cannot generate \"From:\" line without a valid username.");
